@@ -8,6 +8,8 @@ from typing import Callable, Dict, Iterable, List, Optional, Tuple, Union
 import async_retriever as ar
 import pandas as pd
 import pygeoutils as geoutils
+import pyproj
+import rioxarray  # noqa: F401
 import xarray as xr
 from pygeoogc import ServiceError, ServiceURL
 from pygeoogc import utils as ogcutils
@@ -26,12 +28,14 @@ MAX_CONN = 10
 def get_bycoords(
     coords: Tuple[float, float],
     dates: Union[Tuple[str, str], Union[int, List[int]]],
-    crs: str = DEF_CRS,
+    crs: Union[str, pyproj.CRS] = DEF_CRS,
     variables: Optional[Union[Iterable[str], str]] = None,
     region: str = "na",
     time_scale: str = "daily",
     pet: Optional[str] = None,
     pet_params: Optional[Dict[str, float]] = None,
+    snow: bool = False,
+    snow_params: Optional[Dict[str, float]] = None,
     ssl: Union[SSLContext, bool, None] = None,
     expire_after: float = EXPIRE,
     disable_caching: bool = False,
@@ -76,6 +80,14 @@ def get_bycoords(
     pet_params : dict, optional
         Model-specific parameters as a dictionary that is passed to the PET function.
         Defaults to ``None``.
+    snow : bool, optional
+        Compute snowfall from precipitation and minimum temperature. Defaults to ``False``.
+    snow_params : dict, optional
+        Model-specific parameters as a dictionary that is passed to the snowfall function.
+        These parameters are only used if ``snow`` is ``True``. Two parameters are required:
+        ``t_rain`` (deg C) which is the threshold for temperature for considering rain and
+        ``t_snow`` (deg C) which is the threshold for temperature for considering snow.
+        The default values are ``{'t_rain': 2.5, 't_snow': 0}``.
     ssl : bool or SSLContext, optional
         SSLContext to use for the connection, defaults to None. Set to False to disable
         SSL cetification verification.
@@ -108,7 +120,7 @@ def get_bycoords(
     ----------
     .. footbibliography::
     """
-    daymet = Daymet(variables, pet, time_scale, region)
+    daymet = Daymet(variables, pet, snow, time_scale, region)
     daymet.check_dates(dates)
 
     if isinstance(dates, tuple):
@@ -155,18 +167,24 @@ def get_bycoords(
 
     if pet is not None:
         clm = potential_et(clm, coords, method=pet, params=pet_params)
+
+    if snow:
+        params = {"t_rain": 2.5, "t_snow": 0} if snow_params is None else snow_params
+        clm = daymet.separate_snow(clm, **params)
     return clm
 
 
 def get_bygeom(
     geometry: Union[Polygon, MultiPolygon, Tuple[float, float, float, float]],
     dates: Union[Tuple[str, str], Union[int, List[int]]],
-    crs: str = DEF_CRS,
+    crs: Union[str, pyproj.CRS] = DEF_CRS,
     variables: Optional[Union[Iterable[str], str]] = None,
     region: str = "na",
     time_scale: str = "daily",
     pet: Optional[str] = None,
     pet_params: Optional[Dict[str, float]] = None,
+    snow: bool = False,
+    snow_params: Optional[Dict[str, float]] = None,
     ssl: Union[SSLContext, bool, None] = None,
     expire_after: float = EXPIRE,
     disable_caching: bool = False,
@@ -207,6 +225,14 @@ def get_bygeom(
     pet_params : dict, optional
         Model-specific parameters as a dictionary that is passed to the PET function.
         Defaults to ``None``.
+    snow : bool, optional
+        Compute snowfall from precipitation and minimum temperature. Defaults to ``False``.
+    snow_params : dict, optional
+        Model-specific parameters as a dictionary that is passed to the snowfall function.
+        These parameters are only used if ``snow`` is ``True``. Two parameters are required:
+        ``t_rain`` (deg C) which is the threshold for temperature for considering rain and
+        ``t_snow`` (deg C) which is the threshold for temperature for considering snow.
+        The default values are ``{'t_rain': 2.5, 't_snow': 0}``.
     ssl : bool or SSLContext, optional
         SSLContext to use for the connection, defaults to None. Set to False to disable
         SSL cetification verification.
@@ -235,7 +261,7 @@ def get_bygeom(
     ----------
     .. footbibliography::
     """
-    daymet = Daymet(variables, pet, time_scale, region)
+    daymet = Daymet(variables, pet, snow, time_scale, region)
     daymet.check_dates(dates)
 
     if isinstance(dates, tuple):
@@ -243,6 +269,7 @@ def get_bygeom(
     else:
         dates_itr = daymet.years_tolist(dates)
 
+    crs = ogcutils.validate_crs(crs)
     _geometry = geoutils.geo2polygon(geometry, crs, DEF_CRS)
 
     if not _geometry.intersects(daymet.region_bbox[region]):
@@ -264,7 +291,7 @@ def get_bygeom(
                 io.BytesIO(r)
                 for r in ar.retrieve_binary(
                     urls,
-                    request_kwds=list(kwds),
+                    request_kwds=kwds,
                     max_workers=MAX_CONN,
                     ssl=ssl,
                     disable=disable_caching,
@@ -281,13 +308,11 @@ def get_bygeom(
         )
         raise ServiceError(msg) from ex
 
-    for k, v in daymet.units.items():
-        if k in clm.variables:
-            clm[k].attrs["units"] = v
+    if len(clm.lat.dims) > 2:
+        clm["lat"] = clm.lat.isel(time=0, drop=True)
+        clm["lon"] = clm.lon.isel(time=0, drop=True)
 
-    clm = clm.drop_vars(["lambert_conformal_conic"])
-
-    daymet_crs = " ".join(
+    clm.attrs["crs"] = " ".join(
         [
             "+proj=lcc",
             "+lat_1=25",
@@ -301,43 +326,26 @@ def get_bygeom(
             "+no_defs",
         ]
     )
-    clm.attrs["crs"] = daymet_crs
-    clm.attrs["nodatavals"] = (0.0,)
-    transform, _, _ = geoutils.get_transform(clm, ("y", "x"))
-    clm.attrs["transform"] = transform
-    clm.attrs["res"] = (transform.a, transform.e)
+    clm = clm.rio.write_crs(clm.attrs["crs"], grid_mapping_name="lambert_conformal_conic")
+    clm = geoutils.xarray_geomask(clm, _geometry, DEF_CRS)
+    clm = clm.drop_vars(["spatial_ref"])
+    for v in clm:
+        if "grid_mapping" in clm[v].attrs:
+            _ = clm[v].attrs.pop("grid_mapping")
 
     if pet is not None:
         clm = potential_et(clm, method=pet, params=pet_params)
 
-    if isinstance(clm, xr.Dataset):
-        for v in clm:
-            clm[v].attrs["crs"] = daymet_crs
-            clm[v].attrs["nodatavals"] = (0.0,)
-
-    masked: xr.Dataset = geoutils.xarray_geomask(clm, _geometry, DEF_CRS)
-    return masked
+    if snow:
+        params = {"t_rain": 2.5, "t_snow": 0} if snow_params is None else snow_params
+        clm = daymet.separate_snow(clm, **params)
+    return clm
 
 
 def _get_filename(
     region: str,
 ) -> Dict[int, Callable[[str], str]]:
-    """Generate an iterable URL list for downloading Daymet data.
-
-    Parameters
-    ----------
-    region : str
-        Region in the US. Acceptable values are:
-
-        * na: Continental North America
-        * hi: Hawaii
-        * pr: Puerto Rico
-
-    Returns
-    -------
-    generator
-        An iterator of generated URLs.
-    """
+    """Get correct filenames based on region and variable of interest."""
     return {
         1840: lambda v: f"daily_{region}_{v}",
         1855: lambda v: f"{v}_monttl_{region}" if v == "prcp" else f"{v}_monavg_{region}",
@@ -350,7 +358,7 @@ def _coord_urls(
     coord: Tuple[float, float],
     region: str,
     variables: Iterable[str],
-    dates: List[Tuple[pd.DatetimeIndex, pd.DatetimeIndex]],
+    dates: List[Tuple[pd.Timestamp, pd.Timestamp]],
 ) -> List[List[Tuple[str, Dict[str, Dict[str, str]]]]]:
     """Generate an iterable URL list for downloading Daymet data.
 
@@ -412,7 +420,7 @@ def _gridded_urls(
     bounds: Tuple[float, float, float, float],
     region: str,
     variables: Iterable[str],
-    dates: List[Tuple[pd.DatetimeIndex, pd.DatetimeIndex]],
+    dates: List[Tuple[pd.Timestamp, pd.Timestamp]],
 ) -> List[Tuple[str, Dict[str, Dict[str, str]]]]:
     """Generate an iterable URL list for downloading Daymet data.
 
